module PuppetLocal
  class Command::Install < Command

    option '--debug', :flag, 'enable puppet debug mode'
    parameter 'hiera_file', 'hiera configuration file', :attribute_name => :hiera_path

    def execute
      puts 'Setting up environment...'
      setup
      create_puppet_hiera_config
      puts 'Checking out git-modules...'
      clone_git_modules
      puts 'Applying catalog...'
      apply
      print on_green ' '
      puts ' Puppet apply succeeded'
    end

    def setup
      raise '`' + @hiera_path + '` does not exist' unless File.exists? @hiera_path
      FileUtils.mkdir_p local_path
      file = File.new(hiera_path_absolute, 'r')
      @hiera = JSON.parse file.read
      file.close
    end

    def create_puppet_hiera_config
      hiera_config = {
          :backends => ['json'],
          :hierarchy => [File.basename(@hiera_path).sub(/\.json$/, '')],
          :json => {
              :datadir => File.dirname(hiera_path_absolute)
          }
      }
      file = File.new(puppet_hiera_config, 'w')
      file.puts(hiera_config.to_yaml.gsub('!ruby/sym ', ':'))
      file.close
    end

    def clone_git_modules
      @puppet_module_paths = []
      if @hiera.include? 'git-modules'
        @hiera['git-modules'].each do |source, version|
          repo_path = local_path Digest::MD5.hexdigest(source)
          if File.directory? repo_path
            repo = Git.open repo_path
            repo.fetch
          else
            repo = Git.clone source, repo_path
          end
          repo.checkout version
          @puppet_module_paths.push repo.dir.path + '/modules'
        end
      end
    end

    def apply
      puppet_code = 'include hiera("classes", [])'
      params = {
          '--debug' => debug?,
          '-e' => puppet_code,
          '--hiera_config' => puppet_hiera_config,
          '--modulepath' => puppet_module_path
      }
      system_command 'puppet apply', params
    end

    def hiera_path_absolute
      File.expand_path @hiera_path
    end

    def local_path(path = '')
      ENV['HOME'] + '/.puppet-local/' + Digest::MD5.hexdigest(hiera_path_absolute) + '/' + path
    end

    def puppet_hiera_config
      local_path 'hiera.yaml'
    end

    def puppet_module_path
      @puppet_module_paths.join(':')
    end

  end
end
